<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\SM\Service\Sm00012000MedicaoFaturamentoService;

class Sm00012000MedicaoFaturamentoController extends SMCrud {
    
    protected $dataSourceName = "fct_medicaofatura_completo";
    protected $medicaoService;
    
    public function __construct($crudService, $environment, Sm00012000MedicaoFaturamentoService $medicaoService) {
        parent::__construct($crudService, $environment);
        $this->medicaoService = $medicaoService;
    }
    
    public function find(Request\Filter $request, Response $response) {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $conditions = $filterCriteria->getConditions();
            
            foreach ($conditions as &$filter) {
                if ($filter["columnName"] == "NRESTRUTURAM_IT") {
                    $novoValue = "_" . implode("_", $filter["value"]) . "_";
                    
                    $filter["operator"] = "=";
                    $filter["value"] = $novoValue;
                }
            }
            unset($filter);
            
            $newRequest = $this->createNewFilterRequestWithCustomConditions($request, $conditions);
            
            parent::find($newRequest, $response);
            $erro = $response->getError();
            if ($erro) {
                throw new \Exception($erro->getMessage(), $erro->getErrorCode());
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), 500));
        }
    }

    /**
    * Faz o cálculo dos itens de acordo com a medida variável
    */
    public function recalcularItensMedicao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrmedicaofatura = $row["NRMEDICAOFATURA"];
            $nrseqcontrato   = $row["NRSEQCONTRATO"];
            
            $this->medicaoService->recalcularItensMedicao($nrmedicaofatura, $nrseqcontrato);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Executa a procedure "FCT_GERAR_PARCELAS_APURACAO".
     */
    public function gerarApuracaoFatura(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrmedicaofatura = $row["NRMEDICAOFATURA"];
            $nrseqcontrato   = $row["NRSEQCONTRATO"];
            
            $this->medicaoService->gerarApuracaoFatura($nrmedicaofatura, $nrseqcontrato);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Executa a procedure "FCT_CANCELAR_MEDICAO".
     */
    public function cancelarMedicao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrmedicaofatura = $row["NRMEDICAOFATURA"];
            $nrseqcontrato   = $row["NRSEQCONTRATO"];
            
            $this->medicaoService->cancelarMedicao($nrmedicaofatura, $nrseqcontrato);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "validaFatura" valida se ja foi gerada parcela.
     */
    public function validaFatura(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrmedicaofatura = $row["NRMEDICAOFATURA"];
            $nrseqcontrato   = $row["NRSEQCONTRATO"];

            $valida = $this->medicaoService->validaFatura($nrmedicaofatura, $nrseqcontrato);
            
            $response->addDataSet(new DataSet("VALIDA", array("QUANTIDADE" => $valida)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
    * Executa a procedure "FCT_CANCELAR_APURA_MEDICAO".
    */
    public function cancelarApuracaoMedicao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrmedicaofatura = $row["NRMEDICAOFATURA"];
            $nrseqcontrato   = $row["NRSEQCONTRATO"];
            
            $this->medicaoService->cancelarApuracaoMedicao($nrmedicaofatura, $nrseqcontrato);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
    * Executa a procedure atualiza a medida variável de acordo com os dados do OM.
    */
    public function atualizarMedVariavelOM(Request\DataSet $request, Response $response) {
        try {
            $parcelas = $request->getDataSet()->getRows()[0];
            
            $this->medicaoService->atualizarMedVariavelOM($parcelas);
            $response->addDataSet(new DataSet("SUCCESS", array("SUCESSO" => true)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
    * Faz o cálculo das medições recebidas
    */
    public function calcularMultiplasMedicoes(Request\DataSet $request, Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $nrmedicaofatura = $row["NRMEDICAOFATURA"];
                $nrseqcontrato   = $row["NRSEQCONTRATO"];
                
                $this->medicaoService->recalcularItensMedicao($nrmedicaofatura, $nrseqcontrato);
            }
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function cancelarMedicoes(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $medicoes        = $row["MEDICOES"];
            $dsjustificacanc = $row["DSJUSTIFICACANC"];
            
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $this->medicaoService->cancelarMedicoes($medicoes, $dsjustificacanc, $nrorg, $cdoperador);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getViewNegociacao(Request\Row $request, Response $response) 
    {
        try {
            $row = $request->getRow();
            $nrparcelasnegcontr   = $row["NRPARCELASNEGCONTR"];
            $data = $this->medicaoService->getViewNegociacao($nrparcelasnegcontr);
            $response->addDataSet(new DataSet("getViewNegociacao", $data));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function excluirParcelaMedicao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrparcelasnegcontr = $row["NRPARCELASNEGCONTR"];
            $nrnegcontrato      = $row["NRNEGCONTRATO"];
            $nritmedicaofatura  = $row["NRITMEDICAOFATURA"];
            $nrmedicaofatura    = $row["NRMEDICAOFATURA"];
            $nrseqcontrato      = $row["NRSEQCONTRATO"];
            $nrorg              = $this->environment->getNrOrg();
            
            $this->medicaoService->excluirParcelaMedicao($nrparcelasnegcontr, $nrnegcontrato, $nritmedicaofatura, $nrmedicaofatura, $nrseqcontrato, $nrorg);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function buscarFaixaPrecoSemMultiplicacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrseqcontrato      = $row["NRSEQCONTRATO"];
            $cdservorca         = $row["CDSERVORCA"];
            $cdtpmedidaserv     = $row["CDTPMEDIDASERV"];
            $cdtpmedidavariavel = $row["CDTPMEDIDAVARIAVEL"];
            $qtmedida           = $row["QTMEDIDA"];
            $nrorg              = $this->environment->getNrOrg();
            
            $data = $this->medicaoService->buscarFaixaPrecoSemMultiplicacao($nrseqcontrato, $cdservorca, $cdtpmedidaserv, $cdtpmedidavariavel, $qtmedida, $nrorg);
            $response->addDataSet(new DataSet("buscarFaixaPrecoSemMultiplicacao", array("VRPRECOVARIACAO" => $data)));
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}

?>